package com.wechi.adweb.bridge.google.gtm.controller;

import com.wechi.adweb.bridge.common.APIStatus;
import com.wechi.adweb.bridge.common.BaseController;
import com.wechi.adweb.bridge.common.OpenAPIRequest;
import com.wechi.adweb.bridge.common.OpenAPIResponse;
import com.wechi.adweb.bridge.exception.BadRequestException;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.gtm.dto.GTMAccountDTO;
import com.wechi.adweb.bridge.google.gtm.dto.GTMContainerDTO;
import com.wechi.adweb.bridge.google.gtm.service.GTMService;
import com.wechi.adweb.bridge.util.JsonUtils;

import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author wfansh
 */
@Slf4j
@RestController
@RequestMapping("/api/google/gtm")
public class GTMController extends BaseController {

    @Autowired private GTMService gtmService;

    @PostMapping("/accounts/list")
    @ResponseBody
    public OpenAPIResponse<List<GTMAccountDTO>> listGTMAccounts() throws DataException {
        long start = System.currentTimeMillis();
        log.info("****** listGTMAccounts() ******");

        List<GTMAccountDTO> gtmAccounts = gtmService.listGTMAccounts();

        log.info("****** listGTMAccounts() ****** duration = {} seconds", getElapsedSeconds(start));
        return OpenAPIResponse.<List<GTMAccountDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(gtmAccounts)
                .build();
    }

    @PostMapping("/containers/list")
    @ResponseBody
    public OpenAPIResponse<List<GTMContainerDTO>> listGTMContainers(
            @RequestBody OpenAPIRequest<String> apiRequest)
            throws BadRequestException, DataException {
        long start = System.currentTimeMillis();
        log.info("****** listGTMContainers() ****** apiRequest = {}", JsonUtils.toJson(apiRequest));
        String accountResourceName = apiRequest.getData();

        // 1. Validates the request parameters.
        if (StringUtils.isEmpty(accountResourceName)) {
            throw new BadRequestException(apiRequest);
        }

        // 2. Executes the API request.
        List<GTMContainerDTO> gtmContainers =
                gtmService.listGTMContainers(
                        // Converts to resource name if the account ID is provided by mistake.
                        GTMAccountDTO.toResourceName(accountResourceName));
        log.info(
                "****** listGTMContainers() ****** duration = {} seconds",
                getElapsedSeconds(start));
        return OpenAPIResponse.<List<GTMContainerDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(gtmContainers)
                .build();
    }
}
